package gssim.db.mysql;

import gssim.db.dao.data.TextProperty;
import gssim.db.find.Finder;
import gssim.db.utils.PreparedStatementAdapter;
import gssim.db.utils.ResultSetAdapter;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.apache.commons.logging.LogFactory;

import com.mysql.jdbc.NotImplemented;

public class TextPropertyDAO extends AbstractDAO implements
		gssim.db.dao.TextPropertyDAO {

	protected TextPropertyDAO(Connection conn) throws SQLException {
		super(conn);
		log = LogFactory.getLog(TextPropertyDAO.class);
		setRelationName("text_property");
	}

	@Override
	protected PreparedStatementAdapter createDeleteStmt() throws SQLException {
		return null;
	}

	@Override
	protected PreparedStatementAdapter createFindStmt() throws SQLException {
		String sql = "SELECT * FROM text_property WHERE"
				+ " (config_id = ? OR ISNULL(?))";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createInsertStmt() throws SQLException {
		String sql = "INSERT INTO text_property VALUES(?, ?, ?)";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));

		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createSelectStmt() throws SQLException {
		String sql = "SELECT * FROM text_property WHERE config_id = ? AND name = ?";
		PreparedStatementAdapter stmt = new PreparedStatementAdapter(this
				.getConnection().prepareStatement(sql));
		return stmt;
	}

	@Override
	protected PreparedStatementAdapter createUpdateStmt() throws SQLException {
		return null;
	}

	@Override
	public boolean delete(TextProperty obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public void fillObjectFromResultSet(TextProperty obj, ResultSet result)
			throws SQLException {
		throw new NotImplemented();

	}

	@Override
	public int fillStatementWithObject(PreparedStatement stmt,
			TextProperty obj, int offset) throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public List<TextProperty> find(Finder finder) throws SQLException {

		PreparedStatementAdapter stmt = getFindStmt();
		List<TextProperty> list = new ArrayList<TextProperty>();

		stmt.setInt(1, finder.getInteger("configId"));
		stmt.setInt(2, finder.getInteger("configId"));

		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		while (result.next()) {
			TextProperty obj = new TextProperty();

			Integer configId = result.getIntObj("config_id");
			String name = result.getString("name");
			String value = result.getString("value");

			obj.setConfigId(configId);
			obj.setName(name);
			obj.setValue(value);

			list.add(obj);
		}

		return list;
	}

	@Override
	public int insert(TextProperty obj, boolean commit) throws SQLException {

		PreparedStatementAdapter stmt = getInsertStmt();

		stmt.setInt(1, obj.getConfigId());
		stmt.setString(2, obj.getName());
		stmt.setString(3, obj.getValue());

		int result = stmt.executeUpdate();

		if (result == 1) {
			if (commit)
				commit();
			logSuccess("insert");
		} else
			logError("insert", result, obj);

		return result;
	}

	@Override
	public int[] insertBatch(List<TextProperty> objList, boolean commit)
			throws SQLException {
		throw new NotImplemented();
	}

	@Override
	public TextProperty select(TextProperty obj) throws SQLException {

		PreparedStatementAdapter stmt = getSelectStmt();
		stmt.setInt(1, obj.getConfigId());
		stmt.setString(2, obj.getName());
		ResultSetAdapter result = new ResultSetAdapter(stmt.executeQuery());

		result.next();
		boolean first = result.isFirst();
		boolean last = result.isLast();

		if (first && last) {

			obj.setValue(result.getString("value"));
			return obj;
		}

		// no reservation found with specified task_id and unit_id
		if (!first && !last) {
			return null;
		}

		// no idea how this may happen (only if id is not primary key)
		throw new SQLException("Select operation in relation "
				+ getRelationName() + " returned more than one record");
	}

	@Override
	public boolean update(TextProperty obj, boolean commit) throws SQLException {
		throw new NotImplemented();
	}

}