package gssim.gssimstudio.editor.gssim.minimap
{
	import com.degrafa.Surface;
	import com.degrafa.core.utils.ColorKeys;
	
	import flash.events.Event;
	import flash.events.MouseEvent;
	import flash.geom.Matrix;
	import flash.geom.Rectangle;
	
	import gssim.gssimstudio.editor.gssim.editors.topology.ObjectCanvas;
	import gssim.gssimstudio.editor.gssim.topology.GenderObject;
	import gssim.gssimstudio.editor.gssim.topology.ResourceObject;
	import gssim.gssimstudio.editor.gssim.topology.UserObject;
	
	import mx.graphics.SolidColor;
	import mx.graphics.SolidColorStroke;
	
	import spark.components.Group;
	import spark.primitives.Ellipse;
	import spark.primitives.Rect;
	
	/**
	 * Defines minimap behaviours
	 */
	public class Minimap extends Surface
	{
		private var objects : Object; // map id -> geometry group
				
		private var _scaleX : Number = 16;
		private var _scaleY : Number = 16;
		
		private var numberOfScreensVertical : Number;
		private var numberOfScreensHorizontal : Number;
		private var currentMaxWidth : Number;
		private var currentMaxHeight : Number;
		
		private var visibleRectangleGroup : Group;
		private var rectangle : Rect;
		
		private var targetCanvas : ObjectCanvas;
		
		public function Minimap()
		{
			objects = new Object();
			visibleRectangleGroup = new Group(); 
			rectangle = new Rect();
			rectangle.x = 0;
			rectangle.y = 0;
			rectangle.width = 50;
			rectangle.height = 50;
			rectangle.radiusX = 5;
			var rectangleFill : SolidColor = new SolidColor(ColorKeys.BLACK);
			rectangleFill.alpha = 0.0;
			rectangle.fill = rectangleFill;
			var rectangleStroke : SolidColorStroke = new SolidColorStroke(ColorKeys.DARKGREEN);
			rectangleStroke.weight = 1;
			rectangle.stroke = rectangleStroke;
			visibleRectangleGroup.addElement(rectangle);
			this.addChild(visibleRectangleGroup);
			visibleRectangleGroup.addEventListener(MouseEvent.MOUSE_MOVE, mouseMoveOnVisibleRectangleGroupEventHandler);
			visibleRectangleGroup.addEventListener(MouseEvent.MOUSE_OUT, mouseOutOnVisibleRectangleGroupEventHandler);
		}
		
		/**
		 * Initialize the map
		 */
		public function init(target : ObjectCanvas) : void
		{
			targetCanvas = target;
			scaleChanged();
			target.addEventListener(ObjectCanvas.CANVAS_MOVED, targetMovedEventHandler);
		}
		
		/**
		 * The corresponding canvas was moved by mouse drag
		 */
		private function targetMovedEventHandler(event : Event) : void
		{
			updateVisibleRectanglePosition();
		}

		/**
		 * Stops minimap drag when mouse left the visible box area
		 */
		private function mouseOutOnVisibleRectangleGroupEventHandler(event : MouseEvent) : void
		{
			visibleRectangleGroup.stopDrag();
		}

		/**
		 * Move visible rectangle with mouse drag behaviour
		 */
		private function mouseMoveOnVisibleRectangleGroupEventHandler(event : MouseEvent) : void
		{
			if (event.buttonDown)
			{
				visibleRectangleGroup.startDrag(false, new Rectangle(0,0,this.width - rectangle.width, this.height - rectangle.height));
				updateCanvasPosition();
			}
			else visibleRectangleGroup.stopDrag();
		}

		/**
		 * Update the position of corresponding canvas based on visible rectangle
		 */
		private function updateCanvasPosition() : void
		{
			targetCanvas.surface.x = -visibleRectangleGroup.x * currentMaxWidth / this.width;
			targetCanvas.surface.y = -visibleRectangleGroup.y * currentMaxHeight / this.height;			
		}
		
		/**
		 * Update position of visible rectangle on the minimap
		 */
		private function updateVisibleRectanglePosition() : void
		{
			visibleRectangleGroup.x = -targetCanvas.surface.x * this.width / currentMaxWidth;
			visibleRectangleGroup.y = -targetCanvas.surface.y * this.height / currentMaxHeight;
		}

		/**
		 * Update object position on the minimap
		 */
		public function UpdateObject(object : GenderObject, wasDeleted : Boolean = false) : void
		{
			var minimapObject : Group = null;
			if (objects.hasOwnProperty(object.id))
			{
				minimapObject = objects[object.id];
				if (wasDeleted)
				{
					this.removeChild(minimapObject);
					minimapObject.graphics.clear();
					objects[object.id] = null;
				}
				else
				{
					minimapObject.x = (object.x + object.width/2) / _scaleX;
					minimapObject.y = (object.y + object.height/2) / _scaleY;
				}
			}
			else
			{
				minimapObject = new Group();
				objects[object.id] = minimapObject;
				var circle : Ellipse = new Ellipse();
				circle.x = -10;
				circle.y = -10;
				circle.width = 6;
				circle.height = 6;
				var _lineStroke : SolidColorStroke = new SolidColorStroke(ColorKeys.BLUE);
	            if (object is ResourceObject) _lineStroke.color = ColorKeys.GREEN;
	            if (object is UserObject) _lineStroke.color = ColorKeys.RED;
	            _lineStroke.weight = 2;
	            circle.stroke = _lineStroke;
				minimapObject.addElement(circle);
				minimapObject.x = (object.x + object.width/2) / _scaleX;
				minimapObject.y = (object.y + object.height/2) / _scaleY;
				this.addChild(minimapObject);
			}
		}
		
		/**
		 * Handle canvas size change
		 */
		public function scaleChanged() : void
		{
			var matrix : Matrix = targetCanvas.surface.transform.matrix;
			numberOfScreensHorizontal = (targetCanvas.maximumWidth * matrix.a) / targetCanvas.width;
			numberOfScreensVertical = (targetCanvas.maximumHeight * matrix.d) / targetCanvas.height;
			currentMaxWidth = targetCanvas.maximumWidth * matrix.a;
			currentMaxHeight = targetCanvas.maximumHeight * matrix.d;
			rectangle.width = this.width/numberOfScreensHorizontal;
			rectangle.height = this.height/numberOfScreensVertical;
		}
		
		/**
		 * Clean minimap from all objects
		 */
		public function clean() : void
		{
			while (this.graphicsCollection.items.length) {
				this.graphicsCollection.removeItemAt(0);
			}
			while (this.numChildren) {
				this.removeChildAt(0);
			}
			objects = new Object();
			this.addChild(visibleRectangleGroup);
		}
	}
}
