package gssim.gssimstudio.editor.gssim.multiDrag
{
	import gssim.gssimstudio.editor.gssim.topology.GenderObject;
	
	/**
	 * Class for handling multiple object selection and drag.
	 */
	public class MultiDragContainer
	{
		/**
		 * Map containing objects of group
		 */
		private var dragGroup : Object; //map id -> Object
		/**
		 * Starting point of the drag operation
		 */
		private var startDragPositionX : Number;
		private var startDragPositionY : Number;
		/**
		 * Distance counter during mouse drag
		 */
		private var diffX : Number;
		private var diffY : Number;			 
			 
		public function MultiDragContainer()
		{
			empty();
		}

		/**
		 * Add object to the group
		 */
		public function AddObject(object : GenderObject) : void
		{
			if (!dragGroup.hasOwnProperty(object.id))
			{
				dragGroup[object.id] = object;
				object.selected = true;
			}
		}
		
		/**
		 * Check if object is already in group
		 */
		public function Contains(object : GenderObject) : Boolean
		{
			return dragGroup.hasOwnProperty(object.id);
		}

		/**
		 * Empty the group
		 */
		public function empty() : void
		{
			for each (var object : GenderObject in dragGroup)
			{
				object.selected = false;
			}
			dragGroup = new Object();
			diffX = 0;
			diffY = 0;
		}
		
		/**
		 * Set relative start position and reset distance counters (diffX, diffY)
		 */
		public function setStartPosition(object : GenderObject) : void
		{
			startDragPositionX = object.x;
			startDragPositionY = object.y;
			diffX = 0;
			diffY = 0;
		}
		
		/**
		 * Move all objects in group
		 */
		public function applyDrag(relative : GenderObject) : void
		{
			for each (var object : GenderObject in dragGroup)
			{
				object.dispatchMoveAction();
				if (object == relative) continue;
				object.x += relative.x - startDragPositionX - diffX;
				object.y += relative.y - startDragPositionY - diffY;
			}
			diffX = relative.x - startDragPositionX;
			diffY = relative.y - startDragPositionY;
		}

	}
}
