package gssim.gssimstudio.editor.gssim.resources.file
{
	import flash.events.Event;
	import flash.net.FileReference;
	import flash.utils.ByteArray;
	import flash.utils.Dictionary;
	
	import gssim.gssimstudio.editor.gssim.editors.topology.ObjectCanvas;
	import gssim.gssimstudio.editor.gssim.resources.ComputingResource;
	import gssim.gssimstudio.editor.gssim.resources.HostParameter;
	import gssim.gssimstudio.editor.gssim.resources.NodeInfo;
	import gssim.gssimstudio.editor.gssim.resources.OtherParameter;
	import gssim.gssimstudio.editor.gssim.resources.Property;
	import gssim.gssimstudio.editor.gssim.resources.QueueParameter;
	import gssim.gssimstudio.editor.gssim.resources.QueueResource;
	import gssim.gssimstudio.editor.gssim.resources.ResourcesBasic;
	import gssim.gssimstudio.editor.gssim.topology.GenderObject;
	import gssim.gssimstudio.editor.gssim.topology.GenderObjectFactory;
	import gssim.gssimstudio.editor.gssim.topology.Graph;
	import gssim.gssimstudio.editor.gssim.topology.GraphItem;
	import gssim.gssimstudio.editor.gssim.topology.ResourceObject;
	import gssim.gssimstudio.editor.gssim.topology.RouterObject;
	import gssim.gssimstudio.editor.gssim.topology.UserObject;
	import gssim.gssimstudio.editor.gssim.topology.layouts.gssimFileLayout;
	import gssim.gssimstudio.editor.gssim.topology.layouts.gssimRadialLayout;
	
	import mx.utils.StringUtil;
	
	import gssim.gssimstudio.editor.script.event.ObjectEvent;
	import gssim.gssimstudio.editor.script.views.ViewsHelper;
	
	/**
	 * Loads resources from file
	 */
	public class resourceReader
	{
		private var fref : FileReference;
		private var _canvas : ObjectCanvas;
		
		public function resourceReader(canvas : ObjectCanvas, autobrowse : Boolean = true)
		{
			_canvas = canvas;
			if (autobrowse)
			{
				fref = new FileReference();
				fref.addEventListener(Event.SELECT, selectHandler);
				fref.addEventListener(Event.COMPLETE, completeHandler);
				fref.browse();
			}
		}
		
		public function selectHandler(event:Event):void
		{
		    fref.load();
		}
		
		private function readProperty(xml : XML) : Property
		{
			var property : Property = new Property();
			if (xml.attributes().length() == 1)
			{
				for each (var type : XML in xml.attributes())
				{
					if (type.name() == "name")
						property.propertyType = type;
				}
			}
			else
				throw Error("Invalid file format");
			
			for each (var param : XML in xml.children())
			{
				if (param.name() == "value")
				{
					property.propertyValue = param;
				}
			}
			return property;
		}
		
		private function readHostParameter(xml : XML) : HostParameter
		{
			var host : HostParameter = new HostParameter();
			if (xml.attributes().length() == 1)
			{
				for each (var type : XML in xml.attributes())
				{
					if (type.name() == "name")
						host.hostParameterType = type;
				}
			}
			else
				throw Error("Invalid file format");
			
			for each (var param : XML in xml.children())
			{
				if (param.name() == "paramValue")
				{
					host.hostParameterParamValue = param;
				}
				if (param.name() == "property")
				{
					host.properties.addItem(readProperty(param));
				}
			}
			return host;
		}
		
		private function readOtherParameter(xml : XML) : OtherParameter
		{
			var other : OtherParameter = new OtherParameter();
			if (xml.attributes().length() == 1)
			{
				for each (var type : XML in xml.attributes())
				{
					if (type.name() == "name")
						other.otherParameterType = type;
				}
			}
			else
				throw Error("Invalid file format");
			
			for each (var param : XML in xml.children())
			{
				if (param.name() == "paramValue")
				{
					other.otherParameterValue = param;
				}
			}
			return other;
		}
		
		private function readQueueParameter(xml : XML) : QueueParameter
		{
			var queue : QueueParameter = new QueueParameter();
			if (xml.attributes().length() == 1)
			{
				for each (var type : XML in xml.attributes())
				{
					if (type.name() == "name")
						queue.queueParameterType = type;
				}
			}
			else
				throw Error("Invalid file format");
			
			for each (var param : XML in xml.children())
			{
				if (param.name() == "paramValue")
				{
					queue.queueParameterValue = param;
				}
			}
			return queue;
		}
		
		private function readComputingResource(xml : XML) : ComputingResource
		{
			var res : ComputingResource = new ComputingResource();
			//read name
			if (xml.attributes().length() == 1)
			{
				for each (var paramName : XML in xml.attributes())
				{
					if (paramName.name() == "resourceId")
						res.resourceId = paramName;
				}
			}
			//read host&other parameters
			for each (var param : XML in xml.machineParameters.children())
			{
				if (param.name() == "hostParameter")
				{
					res.hostParameters.addItem(readHostParameter(param));
				}
				if (param.name() == "otherParameter")
				{
					res.otherParameters.addItem(readOtherParameter(param));
				}
			}
			if (xml.hasOwnProperty("nodeInfo"))
			{
				var node : NodeInfo = new NodeInfo();
				for each (var info : XML in xml.nodeInfo.children())
				{
					if (info.name() == "frontendResourceIdRef")
					{
						node.frontendId = info;
					}
					if (info.name() == "queueResourceIdRef")
					{
						node.queueId = info;
					}
				}
				res.nodeInfo = node;
			}
			return res;
		}
		
		private function readQueueResource(xml : XML) : QueueResource
		{
			var res : QueueResource = new QueueResource();
			//read name
			if (xml.attributes().length() == 1)
			{
				for each (var paramName : XML in xml.attributes())
				{
					if (paramName.name() == "resourceId")
						res.resourceId = paramName;
				}
			}
			//read queue parameters
			for each (var param : XML in xml.queueParameters.children())
			{
				if (param.name() == "queueParameter")
				{
					res.queueParameters.addItem(readQueueParameter(param));
				}
			}
			return res;
		}
		
		public function completeHandler(event:Event):void
		{
			//var resources : ResourcesBasic = new ResourcesBasic();
			var data:ByteArray = fref.data;
			readFromString(data);
			fref = null;
		}
		
		public function readFromString(data:ByteArray):void
		{
			//var resources : ResourcesBasic = new ResourcesBasic();
			var names : Object = new Object(); //map of name->count
			var resources : Object = new Object(); //map of name->computing resource
			var graph : Graph = ObjectCanvas.graph;
			
			var xml : XML;
			xml = XML(data.readUTFBytes(data.bytesAvailable));
			var xmlList : XMLList = xml.resources;
			
			var addedResource : Boolean = false;
			
			for each (var inner_computing_xml : XML in xml.resources.computingResource)
			{
				var computing : ComputingResource = readComputingResource(inner_computing_xml);
				//resources.resources.addItem(computing);
				if (computing.nodeInfo != null)
				{
					var name : String = computing.resourceId.split("_")[0];
					if (!names.hasOwnProperty(name))
					{
						names[name] = 0;
						resources[name] = computing;
					}
					names[name]++;
				}
				else
				{
					if (graph.hasNode(computing.resourceId))
						(graph.nodes[computing.resourceId] as ResourceObject).frontend = computing;
					else
					{
						addedResource = true;
						var manualRes: ResourceObject = GenderObjectFactory.createResource(0,0,94,94);
						manualRes.id = computing.resourceId;
						_canvas.addGenderObject(manualRes);
						(graph.nodes[computing.resourceId] as ResourceObject).frontend = computing;
					}
				}
			}
			
			for (var n : String in names)
			{
				var res : ComputingResource = resources[n];
				if (graph.hasNode(res.nodeInfo.frontendId))
				{
					(graph.nodes[res.nodeInfo.frontendId] as ResourceObject).clusteredResources.addItem(res);
					res.count = names[n];
				}
			}
			
			for each (var inner_queue_xml : XML in xml.resources.queueResource)
			{
				var queue : QueueResource = readQueueResource(inner_queue_xml); 
				//resources.queues.addItem(queue);
				var resId : String = queue.resourceId.split("_")[0];
				if (graph.hasNode(resId))
				{
					(graph.nodes[resId] as ResourceObject).queue = queue;
					(graph.nodes[resId] as ResourceObject).convertToCluster();
				}
			}
						
			//apply layout if new resources were added
			if (addedResource)
			{
				_canvas.forceStopDrag(null);
				_canvas.applyLayout(new gssimFileLayout(_canvas.width));	
			}
			
			//clear reference
			_canvas = null;
			
			//resources.toXMLString();
		}

	}
}