package gssim.gssimstudio.editor.gssim.workload
{
	import gssim.gssimstudio.editor.gssim.errors.AbstractError;
	import gssim.gssimstudio.editor.gssim.errors.ValidableObject;
	import gssim.gssimstudio.editor.gssim.errors.workload.PreferenceParameterError;
	import gssim.gssimstudio.editor.gssim.xml.XMLBasicWrapper;
	
	import mx.collections.ArrayCollection;

	/**
	 * Preference parameters are used in preference and describes more advanced options of preference
	 */
	public class PreferenceParameter extends XMLBasicWrapper implements ValidableObject
	{
		private var properties : Object; // map id->_value
		
		public static const name : String = "<name />";
		public static const endpoint : String = "<endpoint />";
		public static const importance : String = "<importance />";
		public static const optimizationType : String = "<optimizationType />";
		public static const value : String = "<value />";
		
		public function get Name() : String
		{
			return properties[name];
		}
		public function set Name(_value : String) : void
		{
			properties[name] = _value;
		}
		
		public function get Endpoint() : String
		{
			return properties[endpoint];
		}
		public function set Endpoint(_value : String) : void
		{
			properties[endpoint] = _value;
		}
		
		public function get Importance() : RandParams
		{
			return properties[importance];
		}
		public function set Importance(_value : RandParams) : void
		{
			properties[importance] = _value;
		}
		
		public function get OptimizationType() : String
		{
			return properties[optimizationType];
		}
		public function set OptimizationType(_value : String) : void
		{
			properties[optimizationType] = _value;
		}
		
		public function get Value() : RandParams
		{
			return properties[value];
		}
		public function set Value(_value : RandParams) : void
		{
			properties[value] = _value;
		}
		
		public function PreferenceParameter()
		{
			properties = new Object();
		}
		
		public override function getXML(addLabel : Boolean) : XML
		{
			var xml : XML = <parameter />;
			if (addLabel) addStringAttribute(xml, "label", xml.toXMLString(), false);
			appendChildWithProperty(xml, properties, name, true, addLabel);
			appendChildWithProperty(xml, properties, endpoint, false, addLabel);
			appendChildWithXML(xml, properties, importance, true, addLabel);
			appendChildWithProperty(xml, properties, optimizationType, true, addLabel);
			appendChildWithXML(xml, properties, value, true, addLabel);
			return xml;
		}
		
		public function validate() : ArrayCollection
		{
			var ac : ArrayCollection = new ArrayCollection();
			var ae : AbstractError;
			if (Name == null) ac.addItem(new PreferenceParameterError("MissingNameError", this));
			if (Importance == null) ac.addItem(new PreferenceParameterError("MissingImportanceError", this));
			else for each (ae in Importance.validate()) ac.addItem(ae);
			if (OptimizationType == null) ac.addItem(new PreferenceParameterError("MissingOptimizationTypeError", this));
			if (Value == null) ac.addItem(new PreferenceParameterError("MissingValueError", this));
			else for each (ae in Value.validate()) ac.addItem(ae);
			return ac;
		}
		
	}
}
