package gssim.gssimstudio.editor.gssim.workload
{
	import gssim.gssimstudio.editor.gssim.errors.AbstractError;
	import gssim.gssimstudio.editor.gssim.errors.ValidableObject;
	import gssim.gssimstudio.editor.gssim.errors.workload.ParametersError;
	import gssim.gssimstudio.editor.gssim.errors.workload.RandParamsByTagError;
	import gssim.gssimstudio.editor.gssim.xml.XMLBasicWrapper;
	
	import mx.collections.ArrayCollection;
	
	/**
	 * Stores workload configuration properties
	 */
	public class WorkloadBasic extends XMLBasicWrapper implements ValidableObject
	{
		private var properties : Object; // map id->value
		
		/**
		 * Defines start time of the simulation in human readable form. The value should be provided in xsd time format. See www.w3.org for details.
		 * Example:
 		 * 		<SimulationStartTime>2009-01-15T10:00:00</SimulationStartTime>
		 */
		public static const SimulationStartTime : String = "<SimulationStartTime/>";

		/**
		 * Defines number of jobs to be generated. This element is used as an alternative for <SimulatonTime/>.
		 * <JobCount/> is element of type RandParams.
		 * Example shows how to create exactly 100 jobs:
 		 *		<JobCount avg="100" distribution="constant"/>
 		 */
 		public static const JobCount : String = "<JobCount/>";

		/**
		 * Defines length of the simulation. Generator will create number of jobs which can be executed in order during SimulationTime. The value should be provided in xsd duration format. See www.w3.org for details. This element is used as an alternative for <JobCount/>.
		 */
		 public static const SimulationTime : String = "<SimulationTime/>";
		 
		/**
		 * Defines number of tasks in each job.
		 * <TaskCount/> is element of type RandParams.
		 * Example shows how to configure generator to create minimum 1 and maximum 10 tasks in each job. The average number of tasks in job will be 5, with standard deviation 3.0 and normal distribution.
 		 * 		<TaskCount avg="5" min="1" max="10" stdev="3.0" distribution="normal"/>
 		 */
 		 public static const TaskCount : String = "<TaskCount/>";

		/**
		 * Defines length of the task in number of instructions. This value will be translated to the seconds with assumption that task of this length will be executed on a single and the slowest processor. The minimum speed of the processor is fixed as a minimum value of cpuspeed host parameters from resource description. The resource description file is specified by resdesc parameter in *.properties file.
		 * <TaskLength/> is element of type RandParams.
		 * Example shows how to configure generator to create task of minimum 500 and maximum 1500 instructions. The average length of all tasks will be 1000 instructions with standard deviation 500.0 and normal distribution.
 		 * <TaskLength avg="1000" min="500" max="1500" stdev="500.0" distribution="normal"/>
		 * The value of all fields in swf file expressed in seconds are calculated as division of task length in instructions and minimum speed of the single processor. If the minimum speed of the processor is 2, then the value of runtime field in swf file for task of length 764 instructions will be calculated as 764/2 = 382 seconds.
		 */
		 public static const TaskLength : String = "<TaskLength/>";

		/**
		 * Defines number of jobs which have the same submit time. Tasks which belongs to one job have always the same submit time. In swf file, submit time is interpreted as number of seconds after simulation start time.
		 * <JobPackageLength/> is element of type RandParams.
		 */
		 public static const JobPackageLength : String = "<JobPackageLength/>";

		/**
		 * Defines time space between submission of successive jobs. In other words, this is the difference between submission time of two successive jobs. The value is expressed in seconds.
		 * JobInterval is element of type RandParams.
		 */
		 public static const JobInterval : String = "<JobInterval/>";
		 
		 /**
		 * Defines generator which creates <hostParameter/> element in task resource requirements section. See GrmsJobDescriptionSchema.xsd for detail description of task resource requirements.
		 * This element requires attribute named: metric. The value of this attribute will be passed to task <hostParameter/> element as a value of its name attribute. The possible values of <hostParameter/> name attribute and thereby <ComputingResourceHostParameter/> metric attribute are: osname, ostype, puarch, osversion, osrelease, memory, freememory, cpucount, freecpus, cpuspeed, application, diskspace, freediskspace, remoteSubmissionInterface, localResourceManager, hostname. In simulation values of cpucount and memory host parameters are used by default. Others are currently ignored.
		 * <ComputingResourceHostParameter/> is element of type RandParams.
		 */
		 public static const ComputingResourceHostParameters : String = "<ComputingResourceHostParameter/>";
		
		/**
		 * This is the complex element, which was designed to describe section <preferences/> in task requirements. See GrmsJobDescriptionSchema.xsd for detail description of task resource requirements.
		 * Element <preferences/> consist of list of <parameter/> elements. Each parameter must provide <name/>, <importance/>, <optimizationType/> and <value/> elements. <endpoint/> is optional. The values of these elements are passed to the attributes in parameter element in task resource requirements section. Name of the parameter elements and task parameter attributes are the same.
		 * Workload generated only for simulation purpose does not require section <parameter/> in task resource requirements, therefore element <preferences/> my be skipped in xml workload configuration.
		 * Importance and value are elements of type RandParams.
		 */
		 public static const Preferences : String = "<Preferences/>";

		/**
		 * This complex element was designed to describe <ExecutionTime/> section in task description. See GrmsJobDescriptionSchema.xsd for details.
		 * <ExecutionTime/> consists of four child elements which have following interpretation:
		 * 		<execDuration/> - describes user expectation about how long the task is. It differs from prior element <TaskLength/> which defines real length of the task. Value of this element is interpreted as number of instructions, and it will be translated into the seconds in the same way as <TaskLength/> is.
		 * 		<periodStart/> - defines point in time from which task execution can be started. Value of this element is interpreted as number of seconds after <SimulationStartTime/>.
		 * 		<periodEnd/> - defines point in time until task task execution must end. Value of this element is interpreted as number of seconds after <SimulationStartTime/>. It can be used as an alternative for <periodDuration/>.
		 * 		<periodDuration/> - defines number of seconds after which task execution must end. This element can be used as an alternative for <periodEnd/>.
		 * <execDuration/>, <periodStart/>, <periodEnd/> and <periodDuration/> are elements of type RandParams.
		 * <ExecutionTime/> element is optional and it is not compulsory to use it in xml workload configuration. However, if advance reservation will be used, then <periodStart/> and <periodEnd/> (<periodDuration/>) are used as a begin and end time of the reservation.
		 */
		 private static const ExecutionTime : String = "<ExecutionTime/>";
		 public static const ExecDuration : String = "<execDuration/>";
		 public static const PeriodStart : String = "<periodStart/>";
		 public static const PeriodEnd : String = "<periodEnd/>";
		 public static const PeriodDuration : String = "<periodDuration/>";
		
		/**
		 * Returns bindable array collection with stored CRHP
		 */
		public function get CRHP() : ArrayCollection
		{
			return properties[ComputingResourceHostParameters] as ArrayCollection;
		}
		
		/**
		 * Returns bindable array collection with stored preferences
		 */
		public function get Prefs() : ArrayCollection
		{
			return properties[Preferences] as ArrayCollection;
		}
		
		public function WorkloadBasic()
		{
			properties = new Object();
			properties[ComputingResourceHostParameters] = new ArrayCollection();
			properties[Preferences] = new ArrayCollection();
		}
		
		public override function getXML(addLabel : Boolean) : XML
		{
			var xml : XML = <WorkloadConfiguration />;
			if (addLabel) addStringAttribute(xml, "label", xml.toXMLString(), false);
			appendChildWithDate(xml, properties, SimulationStartTime, false, addLabel);
			
			//it's mandatory to fill one of these two fields, but not both
			if (properties[JobCount] != null)
			{
				if (properties[SimulationTime] != null) throw Error("JobCount and SimulationTime cannot be set together!");
				else appendChildWithXML(xml, properties, JobCount, true, addLabel);
			}
			else if (properties[SimulationTime] != null) appendChildWithProperty(xml, properties, SimulationTime, true, addLabel);
			else throw Error("JobCount or SimulationTime must be set!");
			
			appendChildWithXML(xml, properties, TaskCount, true, addLabel);
			appendChildWithXML(xml, properties, TaskLength, true, addLabel);
			appendChildWithXML(xml, properties, JobPackageLength, true, addLabel);
			appendChildWithXML(xml, properties, JobInterval, true, addLabel);
			
			for each (var crhp : ComputingResourceHostParameter in properties[ComputingResourceHostParameters])
			{
				xml.appendChild(crhp.getXML(addLabel));
			}
			
			for each (var pp : Preference in properties[Preferences])
			{
				xml.appendChild(pp.getXML(addLabel));
			}
			
			var xml_inner : XML = XML(ExecutionTime);
			if (addLabel) addStringAttribute(xml_inner, "label", xml_inner.toXMLString(), false);
			appendChildWithXML(xml_inner, properties, ExecDuration, false, addLabel);
			appendChildWithXML(xml_inner, properties, PeriodStart, false, addLabel);
			//one cannot declare both fields
			if (properties[PeriodEnd] != null)
			{
				if (properties[PeriodDuration] != null) throw Error("PeriodEnd and PeriodDuration cannot be set together!");
				else appendChildWithXML(xml_inner, properties, PeriodEnd, false, addLabel);
			}
			else appendChildWithXML(xml_inner, properties, PeriodDuration, false, addLabel);
			
			if (xml_inner.children().length() != 0) xml.appendChild(xml_inner);
			
			//for debug only
			//trace(xml.toXMLString());
			return xml;
		}

		/**
		 * Returns property value given by name
		 */
		public function getProperty(propertyName :String) : Object
		{
			return properties[propertyName];
		}
		
		/**
		 * Sets property value given by name
		 */
		public function setProperty(propertyName :String, value : Object) : void
		{
			properties[propertyName] = value;
		}
		
		/**
		 * Returns simulation start time property as Date
		 */
		public function get getSimulationStartTime() : Date
		{
			return properties[SimulationStartTime] as Date;
		}
		
		public function validate() : ArrayCollection
		{
			var ac : ArrayCollection = new ArrayCollection();
			var ae : AbstractError;
			if (properties[JobCount] != null)
			{
				if (properties[SimulationTime] != null)
					ac.addItem(new AbstractError("CannotBeSetTogetherError", "JobCount, SimulationTime"));
				else
					for each (ae in properties[JobCount].validate()) ac.addItem(ae);
			}
			else if (properties[SimulationTime] == null) ac.addItem(new AbstractError("OneMustBeSetError", "JobCount, SimulationTime"));
			if (properties[JobInterval] == null) ac.addItem(new RandParamsByTagError("MissingError", JobInterval));
			else for each (ae in properties[JobInterval].validate()) ac.addItem(ae);
			if (properties[JobPackageLength] == null) ac.addItem(new RandParamsByTagError("MissingError", JobPackageLength));
			else for each (ae in properties[JobPackageLength].validate()) ac.addItem(ae);
			if (properties[TaskCount] == null) ac.addItem(new RandParamsByTagError("MissingError", TaskCount));
			else for each (ae in properties[TaskCount].validate()) ac.addItem(ae);
			if (properties[TaskLength] == null) ac.addItem(new RandParamsByTagError("MissingError", TaskLength));
			else for each (ae in properties[TaskLength].validate()) ac.addItem(ae);
			
			for each (var pp : Preference in properties[Preferences])
			{
				for each (ae in pp.validate()) ac.addItem(ae);
			}
			
			//one cannot declare both fields
			if (properties[PeriodEnd] != null)
			{
				if (properties[PeriodDuration] != null) ac.addItem(new AbstractError("CannotBeSetTogetherError", "PeriodEnd, PeriodDuration"));
			}
			
			if (properties[ExecDuration] != null) for each (ae in properties[ExecDuration].validate()) ac.addItem(ae);
			if (properties[PeriodStart] != null) for each (ae in properties[PeriodStart].validate()) ac.addItem(ae);
			if (properties[PeriodEnd] != null) for each (ae in properties[PeriodEnd].validate()) ac.addItem(ae);
			if (properties[PeriodDuration] != null) for each (ae in properties[PeriodDuration].validate()) ac.addItem(ae);
			
			//check for reference id errors
			var referencing : ArrayCollection = new ArrayCollection();
			var listOfIds : ArrayCollection = new ArrayCollection();
			for each (var rand : Object in properties)
			{
				if (rand is RandParams)
				{
					addToLists(listOfIds, referencing, rand as RandParams, ac);
				}
			}
			for each (var cpar : ComputingResourceHostParameter in CRHP)
			{
				addToLists(listOfIds, referencing, cpar.value, ac);
				addToLists(listOfIds, referencing, cpar.range.min, ac);
				addToLists(listOfIds, referencing, cpar.range.max, ac);
			}
			for each (var r : RandParams in referencing)
			{
				if (!listOfIds.contains(r.params.RefElementId)) ac.addItem(new ParametersError("UnknownReferenceError", r.params));
				else if (r.params.RefElementId == r.params.Id) ac.addItem(new ParametersError("SelfReferenceError", r.params));
				//TODO: detect complex cycle references
			}
			
			return ac;
		}
		
		/**
		 * Add given id to lists for id reference errors validation
		 */
		private function addToLists(ids : ArrayCollection, refs : ArrayCollection, rand : RandParams, errorList : ArrayCollection) : void
		{
			if (rand == null) return;
			if (rand.params.Id != null && rand.params.Id != "")
			{
				if (ids.contains(rand.params.Id)) errorList.addItem(new ParametersError("DoubleIDError", rand.params));
				else ids.addItem(rand.params.Id);
			}
			if (rand.params.RefElementId != null && rand.params.RefElementId != "") refs.addItem(rand);
		}
	}
}
