package gssim.gssimstudio.editor.gssim.workload.swf
{
	import flash.utils.ByteArray;
	
	import mx.collections.ArrayCollection;
	
	/**
	 * Stores original SWF workload file with basic statistics
	 */
	public class WorkloadSWF
	{
		/**
		 * Array with calculated statistics
		 */
		[Bindable]
		public var statistics : ArrayCollection;
		/**
		 * Array with defined jobs
		 */
		[Bindable]
		public var jobs : ArrayCollection;
		/**
		 * String containing headers of the file
		 */
		[Bindable]
		public var headers : String;
		
		/**
		 * This constructor takes data from ByteArray and converts to proper structure
		 */
		public function WorkloadSWF(dataFromFile : ByteArray = null)
		{
			jobs = new ArrayCollection();
			statistics = new ArrayCollection();
			headers = "";
			
			if (dataFromFile == null) return;

			//analyze the file content			
			var data:ByteArray = new ByteArray();
			data.writeBytes(dataFromFile);
			data.position = 0;
		
			//initialize statistics variables
			var longestRun : Number = 0; // wait time + run time
			var avgRunTime : Number = 0; // only run time
			var maxReqTime : Number = 0; // maximum requested amount of proc time [s]
			var maxReqMem : Number = 0; // maximum requested amount of memory [kb per proc]
		
			var array : Array;
			//will this be the same on unix ?
			array = data.readUTFBytes(data.bytesAvailable).split('\n');

			for (var i : Number = 0; i < array.length; i++)
			{
				var line : String = array[i];
				if (line == null) continue;
				if (line.search(";") != -1)
				{
					headers += line.replace(";", "") + "\r";
					continue;
				}
				var laror : Array = line.split(" ");
				var lar : Array = new Array();
				for (var k : Number = 0; k < laror.length; k++)
				{
					if (laror[k] != "") lar.push(laror[k]);
				}
				if (lar.length != 18) continue;
				jobs.addItem(
					{Field1 : Number(lar[0]),
					Field2 : Number(lar[1]),
					Field3 : Number(lar[2]),
					Field4 : Number(lar[3]),
					Field5 : Number(lar[4]),
					Field6 : Number(lar[5]),
					Field7 : Number(lar[6]),
					Field8 : Number(lar[7]),
					Field9 : Number(lar[8]),
					Field10 : Number(lar[9]),
					Field11 : Number(lar[10]),
					Field12 : Number(lar[11]),
					Field13 : Number(lar[12]),
					Field14 : Number(lar[13]),
					Field15 : Number(lar[14]),
					Field16 : Number(lar[15]),
					Field17 : Number(lar[16]),
					Field18 : Number(lar[17])});
				//calculate statistics
				var obj : Object = jobs.getItemAt(jobs.length-1);
				if (longestRun < obj.Field3 + obj.Field4) longestRun = obj.Field3 + obj.Field4;
				if (maxReqMem < obj.Field10) maxReqMem = obj.Field10;
				if (maxReqTime < obj.Field9) maxReqTime = obj.Field9;
				avgRunTime += obj.Field4;
			}
			
			//update statistics
			statistics.addItem({name : "JobCount", value : jobs.length});
			statistics.addItem({name : "LongestRun", value : longestRun});
			statistics.addItem({name : "AvgRunTime", value : avgRunTime / jobs.length});
			statistics.addItem({name : "MaxReqMem", value : maxReqMem});
			statistics.addItem({name : "MaxReqTime", value : maxReqTime});
		}

	}
}
